==> aolai-1.3.5-sources.jar: cn/ps1/aolai/service/RedisService.java <==
package cn.ps1.aolai.service;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import cn.ps1.aolai.entity.User;
import cn.ps1.aolai.utils.ConfUtil;
import cn.ps1.aolai.utils.Const;
import cn.ps1.aolai.utils.Digest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import redis.clients.jedis.Jedis;
import redis.clients.jedis.ShardedJedis;
import redis.clients.jedis.ShardedJedisPool;

/**
 * 针对Redis的增删改查及Token相关基础业务操作
 *
 * @author Fulin
 * @version 1.0
 * @since 1.7 $Date: 2017.6.17
 */

@Service
public class RedisService {
	
	private static Logger LOG = LoggerFactory.getLogger(RedisService.class);

	@Autowired
	private ShardedJedisPool jedisPool;

	/** 控制同一账户多人同时登录，未配置默认为仅限单用户登录使用 */
	private boolean singleUser() {
		return !ConfUtil.IS_MULTI_LOGIN;
	}

	/**
	 * 新建用户Token，再写入Cookies并返回前端
	 */
	public Map<String, String> newToken(String userId, String spec) {
		Map<String, String> token = new HashMap<>();
		token.put(ConfUtil.CERTID, spec); // 32位证书
		token.put(ConfUtil.TOKEN, Digest.uuid8()); // 8位时间戳字符串
		token.put(User.ID, userId); // 未编码的userId
		// 缓存用户Token，这里的userId为加密前的数据
		setToken(token);
		LOG.debug("newToken...{}", token);
		// 编码处理
		// 写入Cookies为编码处理后的userId信息
		token.put(User.ID, Digest.sm4Encrypt(userId, spec));
		return token;
	}

	/**
	 * 缓存用户Token，控制同一账户多次登录，默认不配置时仅限单一用户登录使用。
	 */
	public void setToken(Map<String, String> token) {
		// 这里仅限单一用户登录，亦或支持多用户同时登录
//		String rdsUid = singleUser() ? encUid : token.get(ConfUtil.TOKEN);
//		hmset(Const.RDS_TOKEN + rdsUid, token, ConfUtil.CACHE_TIME);
		/**
		 * 多次登录仅通过uuid获取token即可，单一登录须校验userId
		 */
		String uuid = token.get(ConfUtil.TOKEN); // 如：uuid=eA8bTjyw
		// t.eA8bTjyw={token}
		hmset(Const.RDS_TOKEN + uuid, token, ConfUtil.CACHE_TIME);
		if (singleUser()) {
			// 如：userId=18666677888
			String userId = token.get(User.ID);
			// 仅单用户时配置了：k.18666677888=eA8bTjyw
			set(Const.RDS_CERT + userId, uuid, ConfUtil.CACHE_TIME);
		}
	}

    /**
     * 验证token是否有效：单用户无法对应多个token，多个token可缓存同一个userid
     * <p>
     * {userId=BMr2LhF2LhF2LhF=,token=QKDMZJXWSFV, ...}
     */
	public Map<String, String> verifyToken(String uuid) {
		/**
		 * 多次登录仅通过uuid获取token即可，单一登录须校验userId
		 */
		String tid = Const.RDS_TOKEN + uuid;
		Map<String, String> token = hmget(tid);
		/**
		 * 正常时可以通过uuid获取到{token}信息
		 */
		if (token != null && expire(tid)) {
			if (singleUser()) {
				// 仅单用户时配置了：k.18666677888=eA8bTjyw
				String uid = Const.RDS_CERT + token.get(User.ID);
				if (uuid.equals(get(uid)) && expire(uid))
					return token;
				LOG.debug("> verifyToken...^rds_uuid");
			} else {
				// 多用户跳过校验：k.18666677888
				return token;
			}
		}
		return null;
	}

    /**
     * 验证token是否有效：单用户无法对应多个token，多个token可缓存同一个userid
     * <p>
     * {userId=BMr2LhF2LhF2LhF=,token=QKDMZJXWSFV, ...}
	 * @deprecated 这个方法已被弃用，并且在未来版本不再支持。
     */
	public Map<String, String> verifyToken(Map<String, String> cookies) {
		return verifyToken(cookies.get(ConfUtil.TOKEN));
	}

	/**
	 * 验证手机动态验证码是否有效
	 * 
	 * @param mobile 手机号码
	 * @param vCode 动态验证码
	 */
	public boolean checkVCode(Object mobile, Object vCode) {
		return exists(Const.RDS_CODE + mobile + vCode);
	}

	/**
	 * 缓存手机动态验证码（默认缓存十分钟）
	 * 
	 * @param mobile 手机号码
	 * @param vCode 动态验证码
	 */
	public void setVCode(Object mobile, Object vCode) {
		String rds_vcode = Const.RDS_CODE + mobile + vCode;
		set(rds_vcode, Const.STR_1, Const.TEN_MM);
	}

	/**
     * 退出登录、清除token信息：{userId,token,certId}
     */
	public void clearToken(Map<String, String> token) {
		// 前面已经验证了token有效性
		LOG.debug("clearToken...{}", token);
		// 只需删除这个基于uuid的token信息即可
		// 基于userId的信息再登录后自动覆盖
		del(Const.RDS_TOKEN + token.get(ConfUtil.TOKEN));
	}

	/**
	 * 缓存当前的路由页面的用户访问权限，主键为：应用、公司、岗位
	 * 
	 * @param key
	 * @param actMap 这里的值都是'1'
	 */
	public void setActionRole(String key, Map<String, String> actMap) {
		hmset(Const.RDS_ROLE + key, actMap, ConfUtil.CACHE_TIME);
	}

	/**
	 * 获取当前路由页面的用户访问权限，主键为：应用、公司、岗位
	 *
	 * @param key 主键
	 * @param act 页面路由
	 * @return String 当前路由的访问权限，有访问权限返回'1'
	 */
	public String getActionRole(String key, String act) {
		String rk = Const.RDS_ROLE + key;
		// 注意这里获取的act值有可能三种状态：null\""\值
		return expire(rk) ? hget(rk, act) : "";
	}

    /**
     * 缓存当前用户信息，以备在访问接口时鉴权时使用
     *
     * @param user 用户信息
     */
    public void setUserInfo(Map<String, String> user) {
		String uid = Const.RDS_USER + user.get(User.ID);
		hmset(uid, user, ConfUtil.CACHE_TIME);
    }

    /**
     * 获取当前登录用户信息，在访问接口时鉴权时使用
     *
     * @param userId 用户编号
     * @return Map 当前用户信息
     */
	public Map<String, String> getUserInfo(Object userId) {
		String rdsKey = Const.RDS_USER + userId;
		return expire(rdsKey) ? hmget(rdsKey) : null;
	}

    /**
     * 设置单个键值，永久有效
     *
     * @param key 主键
     * @param val 值
     */
    public boolean set(String key, String val) {
        return set(key, val, 0); // 返回OK：成功
    }

    /**
     * 设置单个键值，并设置有效时间
     *
     * @param key 主键
     * @param val 值
     * @param sec 有效期（秒）
     */
	public boolean set(String key, String val, int sec) {
		LOG.debug("set...{}={}", key, val);
		boolean result = true;
		ShardedJedis jedis = jedisPool.getResource();
		try {
			// 这里返回值=“OK”，成功
			jedis.setex(key, sec, val);
		} catch (Exception e) {
			result = false;
			LOG.error(e.getMessage());
		} finally {
			close(jedis);
		}
		return result;
	}

	/**
     * 设置map对象，永久有效
     *
     * @param key 主键
     * @param map 对象
     */
    public boolean hmset(String key, Map<String, String> map) {
        return hmset(key, map, 0);
    }

    /**
     * 设置map对象
     *
     * @param key 主键
     * @param map 对象
     * @param sec 有效期（秒）
     * @return String
     */
    public boolean hmset(String key, Map<String, String> map, int sec) {
		LOG.debug("hmset...{}={}", key, map);
		boolean result = true;
		ShardedJedis jedis = jedisPool.getResource();
		try {
			// 这里返回值=“OK”，成功
			jedis.hmset(key, map);
			if (sec > 0)
				expire(key, sec);
		} catch (Exception e) {
			result = false;
			LOG.error("> hmset...{}", e.getMessage());
		} finally {
			close(jedis);
		}
		return result;
    }

    /**
     * 获取map对象
     */
	public Map<String, String> hmget(String key) {
		Map<String, String> result = null;
		ShardedJedis jedis = jedisPool.getResource();
		try {
			result = jedis.hgetAll(key);
			if (result != null && result.isEmpty())
				result = null;
		} catch (Exception e) {
			LOG.error(e.getMessage());
		} finally {
			close(jedis);
		}
		LOG.debug("hmget...{}={}", key, result);
		return result;
	}

    /**
     * 获取map对象单个域的键值
     */
    public String hget(String key, String field) {
        String result = null;
        ShardedJedis jedis = jedisPool.getResource();
        try {
            result = jedis.hget(key, field);
        } catch (Exception e) {
            LOG.error(e.getMessage());
        } finally {
        	close(jedis);
        }
        LOG.debug("hget...{}={}", key, result);
        return result;
    }

    /**
     * 设置map对象单个域的键值，永久有效
     */
    public boolean hset(String key, String field, String val) {
        return hset(key, field, val, 0);
    }

    /**
     * 设置map对象单个域的键值
     */
    public boolean hset(String key, String field, String val, int sec) {
		LOG.debug("hset...{}", key);
		boolean result = true;
		ShardedJedis jedis = jedisPool.getResource();
		try {
			// 这里新建时返回1，覆盖返回0
			jedis.hset(key, field, val);
			if (sec > 0)
				expire(key, sec);
		} catch (Exception e) {
			result = false;
			LOG.error(e.getMessage());
		} finally {
			close(jedis);
		}
		return result;
    }

    /**
     * 获取单个键值
     */
    public String get(String key) {
        String result = null;
        ShardedJedis jedis = jedisPool.getResource();
        try {
            result = jedis.get(key);
        } catch (Exception e) {
            LOG.error(e.getMessage());
        } finally {
        	close(jedis);
        }
		LOG.debug("get...{}={}", key, result);
        return result;
    }

	/**
	 * 根据关键字模糊匹配的结果
	 */
	public List<String> getKeys(String pattern) {
		ShardedJedis jedis = jedisPool.getResource();
		Collection<Jedis> allShards = jedis.getAllShards();
		List<String> keyList = new ArrayList<>();
		for (Jedis j : allShards) {
			// 模糊匹配如：keys + "*"
			Set<String> set = j.keys(pattern);
			keyList.addAll(set);
		}
		close(jedis);
		return keyList;
	}

	/**
     * 根据key删除键值对数据
     */
	public boolean del(String key) {
		Long count = 0L;
		ShardedJedis jedis = jedisPool.getResource();
		try {
			count = jedis.del(key);
		} catch (Exception e) {
			LOG.error(e.getMessage());
		} finally {
			close(jedis);
		}
		return count > 0;
	}

	/**
	 * 批量删除按关键字模糊匹配的结果
	 */
	public boolean delKeys(String pattern) {
		ShardedJedis jedis = jedisPool.getResource();
		Collection<Jedis> allShards = jedis.getAllShards();
		Long count = 0L;
		for (Jedis j : allShards) {
			Set<String> set = j.keys(pattern);
			for (String key : set)
				count += jedis.del(key);
		}
		close(jedis);
		return count > 0;
	}

	/**
	 * 批量删除按关键字模糊匹配的结果
	 */
	public boolean delKeys(List<String> keyList) {
		ShardedJedis jedis = jedisPool.getResource();
		Long count = 0L;
		for (String key : keyList) {
			count += jedis.del(key);
		}
		close(jedis);
		return count > 0;
	}

	/**
     * 根据key删除键值对数据
     */
    public boolean hdel(String key, String[] fields) {
    	Long count = 0L;
        ShardedJedis jedis = jedisPool.getResource();
        try {
        	count = jedis.hdel(key, fields);
        } catch (Exception e) {
            LOG.error(e.getMessage());
        } finally {
        	close(jedis);
        }
        return count > 0;
    }

    /**
     * 判断key是否存在
     */
    public boolean exists(String key) {
        ShardedJedis jedis = jedisPool.getResource();
        try {
            return jedis.exists(key);
        } catch (Exception e) {
            LOG.error(e.getMessage());
        } finally {
        	close(jedis);
        }
        return false;
    }

    /**
     * 设置key的默认N小时有效期，或延续有效期时间
     */
    public boolean expire(String key) {
        return expire(key, ConfUtil.CACHE_TIME);
    }

	/**
	 * 设置key的默认有效期，或延续有效期时间
	 */
	public boolean expire(String key, int sec) {
		long result = 0L;
		ShardedJedis jedis = jedisPool.getResource();
		try {
			// 返回1-成功，0-失败
			result = jedis.expire(key, sec);
		} catch (Exception e) {
			LOG.error("expire...{}", e.getMessage());
			e.printStackTrace();
		} finally {
			close(jedis);
		}
		return result > 0;
	}

	/**
	 * 释放资源
	 * 
	 * @param jedis
	 */
	private void close(ShardedJedis jedis) {
		if (jedis != null)
			jedis.close();
	}

	/**
     * 阻止指定ip访问，指定时间之内（1分钟、30分钟）限制IP访问次数（10次、50次）
     *
     * @param ip 客户端IP地址
     */
    public boolean isBlocked(String ip) {
        int[] num = {Const.ONE_MM, 3 * Const.TEN_MM, 10, 50};
        return isBlocked(ip, num);
    }

    /**
     * 对短时间内尝试登陆的ip次数进行限制
     *
     * @param ip  客户端IP地址
     * @param num 一定时间内及次数限制
     * @return boolean
     */
    public boolean isBlocked(String ip, int[] num) {
        String rk = Const.RDS_DENY + ip;
        Map<String, String> addr = hmget(rk);
        // 记录当前访问时间戳，如：1693814052623
        String time = String.valueOf(new Date().getTime());
        int loop = num.length / 2;
        if (addr == null) {
            // 初次访问，未锁
            addr = new HashMap<>();
            addr.put("locked", "0");
        } else if ("1".equals(addr.get("locked"))) { // 已锁定
            return true;
        } else {
            String[] arr = addr.get("time").split(",");
            int len = arr.length, i = 0;
            while (i < len) {
                long diff = new Date().getTime() - Long.parseLong(arr[i]);
                for (int n = 0; n < loop; n++) {
                    if (diff < num[0]) { // 指定时间之内（1分钟、30分钟）
                        if (len - i < num[n + loop]) // 小于N次
                            break;
                        // N次及以上（10次、50次）
                        addr.put("locked", "1"); // 锁定
                        hmset(rk, addr, loop - 1);
                        return true;
                    }
                }
                i++;
            }
            while (i < len) {
                time = arr[i] + "," + time;
                i++;
            }
        }
        addr.put("time", time);
        hmset(rk, addr, loop - 1); // 循环次数
        return false;
    }

	/**
	 * 生成一个66位国密SM2公钥，并缓存私钥 sec秒后失效
	 * <p>
	 * 仅登录初始化用在了getTicket()中，缓存2分钟
	 */
	public String getSm2PubKey(Object ticket, int sec) {
		Map<String, String> keyPair = Digest.genSm2Pair();
		String pubk = keyPair.get(ConfUtil.PUB_KEY); // 66位公钥
		String prik = keyPair.get(ConfUtil.PRI_KEY); // 64位私钥
		/**
		 * 用了两种方式来缓存私钥（sec秒后失效）<br>
		 * 1.以ticket为key暂存私钥<br>
		 * 2.以临时公钥为key暂存私钥
		 */
		set(Const.RDS_APPID + ticket, prik, sec);
		set(Const.RDS_CERT + pubk, prik, sec);
		/** 返回前端公钥 */
		return pubk;
	}

	/**
	 * 读取临时缓存的64位国密SM2私钥
	 */
//	public String getSm2PriKey(Object key) {
	String getCertKey(Object key) {
		/**
		 * 支持两种方式获取私钥<br>
		 * 1.以ticket为key获取私钥<br>
		 * 2.以临时公钥为key获取私钥（目前使用第2种方式）
		 */
		String prik = get(Const.RDS_APPID + key);
		return prik == null ? get(Const.RDS_CERT + key) : prik;
	}

}

==> aolai-1.3.10-sources.jar: cn/ps1/aolai/service/RedisService.java <==
package cn.ps1.aolai.service;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import cn.ps1.aolai.entity.User;
import cn.ps1.aolai.utils.ConfUtil;
import cn.ps1.aolai.utils.Const;
import cn.ps1.aolai.utils.Digest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import redis.clients.jedis.Jedis;
import redis.clients.jedis.ShardedJedis;
import redis.clients.jedis.ShardedJedisPool;

/**
 * 针对Redis的增删改查及Token相关基础业务操作
 *
 * @author Aolai
 * @version 1.0
 * @since 1.7 $Date: 2017.6.17
 */

@Service
public class RedisService {
	
	private static Logger LOG = LoggerFactory.getLogger(RedisService.class);

	@Autowired
	private ShardedJedisPool jedisPool;

	/** 控制同一账户多人同时登录，未配置默认为仅限单用户登录使用 */
	private boolean singleUser() {
		return !ConfUtil.isMultLogin();
	}

	/**
	 * 新建用户Token，再写入Cookies并返回前端
	 */
	public Map<String, String> newToken(String userId, String spec) {
		Map<String, String> token = new HashMap<>();
		token.put(ConfUtil.CERTID, spec); // 32位证书
		token.put(ConfUtil.TOKEN, Digest.uuid8()); // 8位时间戳字符串
		token.put(User.ID, userId); // 未编码的userId
		// 缓存用户Token，这里的userId为加密前的数据
		setToken(token);
		LOG.debug("newToken...{}", token);
		// 编码处理
		// 写入Cookies为编码处理后的userId信息
		token.put(User.ID, Digest.sm4Encrypt(userId, spec));
		return token;
	}

	/**
	 * 缓存用户Token，控制同一账户多次登录，默认不配置时仅限单一用户登录使用。
	 */
	public void setToken(Map<String, String> token) {
		// 这里仅限单一用户登录，亦或支持多用户同时登录
//		String rdsUid = singleUser() ? encUid : token.get(ConfUtil.TOKEN);
//		hmset(Const.RDS_TOKEN + rdsUid, token, ConfUtil.CACHE_TIME);
		/**
		 * 多次登录仅通过uuid获取token即可，单一登录须校验userId
		 */
		String uuid = token.get(ConfUtil.TOKEN); // 如：uuid=eA8bTjyw
		// t.eA8bTjyw={token}
		hmset(Const.RDS_TOKEN + uuid, token, ConfUtil.cacheTime());
		if (singleUser()) {
			// 如：userId=18666677888
			String userId = token.get(User.ID);
			// 仅单用户时配置了：k.18666677888=eA8bTjyw
			set(Const.RDS_CERT + userId, uuid, ConfUtil.cacheTime());
		}
	}

    /**
     * 验证token是否有效：单用户无法对应多个token，多个token可缓存同一个userid
     * <p>
     * {userId=BMr2LhF2LhF2LhF=,token=QKDMZJXWSFV, ...}
     */
	public Map<String, String> verifyToken(String uuid) {
		/**
		 * 多次登录仅通过uuid获取token即可，单一登录须校验userId
		 */
		String tid = Const.RDS_TOKEN + uuid;
		Map<String, String> token = hmget(tid);
		/**
		 * 正常时可以通过uuid获取到{token}信息
		 */
		if (token != null && expire(tid)) {
			if (singleUser()) {
				// 仅单用户时配置了：k.18666677888=eA8bTjyw
				String uid = Const.RDS_CERT + token.get(User.ID);
				if (uuid.equals(get(uid)) && expire(uid))
					return token;
				LOG.debug("> verifyToken...^rds_uuid");
			} else {
				// 多用户跳过校验：k.18666677888
				return token;
			}
		}
		return null;
	}

    /**
     * 验证token是否有效：单用户无法对应多个token，多个token可缓存同一个userid
     * <p>
     * {userId=BMr2LhF2LhF2LhF=,token=QKDMZJXWSFV, ...}
	 * @deprecated 这个方法已被弃用，并且在未来版本不再支持。
     */
	public Map<String, String> verifyToken(Map<String, String> cookies) {
		return verifyToken(cookies.get(ConfUtil.TOKEN));
	}

	/**
	 * 验证手机动态验证码是否有效
	 * 
	 * @param mobile 手机号码
	 * @param vCode 动态验证码
	 */
	public boolean checkVCode(Object mobile, Object vCode) {
		return exists(Const.RDS_CODE + mobile + vCode);
	}

	/**
	 * 缓存手机动态验证码（默认缓存十分钟）
	 * [时效从配置中取，如果没有配置，则取默认值]
	 * 
	 * @param mobile 手机号码
	 * @param vCode 动态验证码
	 */
	public void setVCode(Object mobile, Object vCode) {
		String rds_vcode = Const.RDS_CODE + mobile + vCode;
		// 不推荐在这里处理
//		String smsExpiredConfig = ConfUtil.getConf("smsCodeExpiredMin");
//		int smsExpired = Const.TEN_MM;
//		if(smsExpiredConfig != null && !"".equals(smsExpiredConfig.trim()) && smsExpiredConfig.length()<=4) {
//		    smsExpired = Integer.parseInt(smsExpiredConfig) * 60;
//		}
//		set(rds_vcode, Const.S_1, smsExpired);
		set(rds_vcode, Const.S_1, ConfUtil.vcodeDue());
	}

	/**
     * 退出登录、清除token信息：{userId,token,certId}
     */
	public void clearToken(Map<String, String> token) {
		// 前面已经验证了token有效性
		LOG.debug("clearToken...{}", token);
		// 只需删除这个基于uuid的token信息即可
		// 基于userId的信息再登录后自动覆盖
		del(Const.RDS_TOKEN + token.get(ConfUtil.TOKEN));
	}

	/**
	 * 缓存当前的路由页面的用户访问权限，主键为：应用、公司、岗位
	 * 
	 * @param key
	 * @param actMap 这里的值都是'1'
	 */
	public void setActionRole(String key, Map<String, String> actMap) {
		hmset(Const.RDS_ROLE + key, actMap, ConfUtil.cacheTime());
	}

	/**
	 * 获取当前路由页面的用户访问权限，主键为：应用、公司、岗位
	 *
	 * @param key 主键
	 * @param act 页面路由
	 * @return String 当前路由的访问权限，有访问权限返回'1'
	 */
	public String getActionRole(String key, String act) {
		String rk = Const.RDS_ROLE + key;
		// 注意这里获取的act值有可能三种状态：null\""\值
		return expire(rk) ? hget(rk, act) : "";
	}

    /**
     * 缓存当前用户信息，以备在访问接口时鉴权时使用
     *
     * @param user 用户信息
     */
    public void setUserInfo(Map<String, String> user) {
		String uid = Const.RDS_USER + user.get(User.ID);
		hmset(uid, user, ConfUtil.cacheTime());
    }

    /**
     * 获取当前登录用户信息，在访问接口时鉴权时使用
     *
     * @param userId 用户编号
     * @return Map 当前用户信息
     */
	public Map<String, String> getUserInfo(Object userId) {
		String rdsKey = Const.RDS_USER + userId;
		return expire(rdsKey) ? hmget(rdsKey) : null;
	}

    /**
     * 设置单个键值，永久有效
     *
     * @param key 主键
     * @param val 值
     */
    public boolean set(String key, String val) {
        return set(key, val, 0); // 返回OK：成功
    }

    /**
     * 设置单个键值，并设置有效时间
     *
     * @param key 主键
     * @param val 值
     * @param sec 有效期（秒）
     */
	public boolean set(String key, String val, int sec) {
		LOG.debug("set...{}={}", key, val);
		boolean result = true;
		ShardedJedis jedis = jedisPool.getResource();
		try {
			// 这里返回值=“OK”，成功
			jedis.setex(key, sec, val);
		} catch (Exception e) {
			result = false;
			LOG.error(e.getMessage());
		} finally {
			close(jedis);
		}
		return result;
	}

	/**
     * 设置map对象，永久有效
     *
     * @param key 主键
     * @param map 对象
     */
    public boolean hmset(String key, Map<String, String> map) {
        return hmset(key, map, 0);
    }

    /**
     * 设置map对象
     *
     * @param key 主键
     * @param map 对象
     * @param sec 有效期（秒）
     * @return String
     */
    public boolean hmset(String key, Map<String, String> map, int sec) {
		LOG.debug("hmset...{}={}", key, map);
		boolean result = true;
		ShardedJedis jedis = jedisPool.getResource();
		try {
			// 这里返回值=“OK”，成功
			jedis.hmset(key, map);
			if (sec > 0)
				expire(key, sec);
		} catch (Exception e) {
			result = false;
			LOG.error("> hmset...{}", e.getMessage());
		} finally {
			close(jedis);
		}
		return result;
    }

    /**
     * 获取map对象
     */
	public Map<String, String> hmget(String key) {
		Map<String, String> result = null;
		ShardedJedis jedis = jedisPool.getResource();
		try {
			result = jedis.hgetAll(key);
			if (result != null && result.isEmpty())
				result = null;
		} catch (Exception e) {
			LOG.error(e.getMessage());
		} finally {
			close(jedis);
		}
		LOG.debug("hmget...{}={}", key, result);
		return result;
	}

    /**
     * 获取map对象单个域的键值
     */
    public String hget(String key, String field) {
        String result = null;
        ShardedJedis jedis = jedisPool.getResource();
        try {
            result = jedis.hget(key, field);
        } catch (Exception e) {
            LOG.error(e.getMessage());
        } finally {
        	close(jedis);
        }
        LOG.debug("hget...{}={}", key, result);
        return result;
    }

    /**
     * 设置map对象单个域的键值，永久有效
     */
    public boolean hset(String key, String field, String val) {
        return hset(key, field, val, 0);
    }

    /**
     * 设置map对象单个域的键值
     */
    public boolean hset(String key, String field, String val, int sec) {
		LOG.debug("hset...{}", key);
		boolean result = true;
		ShardedJedis jedis = jedisPool.getResource();
		try {
			// 这里新建时返回1，覆盖返回0
			jedis.hset(key, field, val);
			if (sec > 0)
				expire(key, sec);
		} catch (Exception e) {
			result = false;
			LOG.error(e.getMessage());
		} finally {
			close(jedis);
		}
		return result;
    }

    /**
     * 获取单个键值
     */
    public String get(String key) {
        String result = null;
        ShardedJedis jedis = jedisPool.getResource();
        try {
            result = jedis.get(key);
        } catch (Exception e) {
            LOG.error(e.getMessage());
        } finally {
        	close(jedis);
        }
		LOG.debug("get...{}={}", key, result);
        return result;
    }

	/**
	 * 根据关键字模糊匹配的结果
	 */
	public List<String> getKeys(String pattern) {
		ShardedJedis jedis = jedisPool.getResource();
		Collection<Jedis> allShards = jedis.getAllShards();
		List<String> keyList = new ArrayList<>();
		for (Jedis j : allShards) {
			// 模糊匹配如：keys + "*"
			Set<String> set = j.keys(pattern);
			keyList.addAll(set);
		}
		close(jedis);
		return keyList;
	}

	/**
     * 根据key删除键值对数据
     */
	public boolean del(String key) {
		Long count = 0L;
		ShardedJedis jedis = jedisPool.getResource();
		try {
			count = jedis.del(key);
		} catch (Exception e) {
			LOG.error(e.getMessage());
		} finally {
			close(jedis);
		}
		return count > 0;
	}

	/**
	 * 批量删除按关键字模糊匹配的结果
	 */
	public boolean delKeys(String pattern) {
		ShardedJedis jedis = jedisPool.getResource();
		Collection<Jedis> allShards = jedis.getAllShards();
		Long count = 0L;
		for (Jedis j : allShards) {
			Set<String> set = j.keys(pattern);
			for (String key : set)
				count += jedis.del(key);
		}
		close(jedis);
		return count > 0;
	}

	/**
	 * 批量删除按关键字模糊匹配的结果
	 */
	public boolean delKeys(List<String> keyList) {
		ShardedJedis jedis = jedisPool.getResource();
		Long count = 0L;
		for (String key : keyList) {
			count += jedis.del(key);
		}
		close(jedis);
		return count > 0;
	}

	/**
     * 根据key删除键值对数据
     */
    public boolean hdel(String key, String[] fields) {
    	Long count = 0L;
        ShardedJedis jedis = jedisPool.getResource();
        try {
        	count = jedis.hdel(key, fields);
        } catch (Exception e) {
            LOG.error(e.getMessage());
        } finally {
        	close(jedis);
        }
        return count > 0;
    }

    /**
     * 判断key是否存在
     */
    public boolean exists(String key) {
        ShardedJedis jedis = jedisPool.getResource();
        try {
            return jedis.exists(key);
        } catch (Exception e) {
            LOG.error(e.getMessage());
        } finally {
        	close(jedis);
        }
        return false;
    }

    /**
     * 设置key的默认N小时有效期，或延续有效期时间
     */
    public boolean expire(String key) {
        return expire(key, ConfUtil.cacheTime());
    }

	/**
	 * 设置key的默认有效期，或延续有效期时间
	 */
	public boolean expire(String key, int sec) {
		long result = 0L;
		ShardedJedis jedis = jedisPool.getResource();
		try {
			// 返回1-成功，0-失败
			result = jedis.expire(key, sec);
		} catch (Exception e) {
			LOG.error("expire...{}", e.getMessage());
			e.printStackTrace();
		} finally {
			close(jedis);
		}
		return result > 0;
	}

	/**
	 * 释放资源
	 * 
	 * @param jedis
	 */
	private void close(ShardedJedis jedis) {
		if (jedis != null)
			jedis.close();
	}

	/**
     * 阻止指定ip访问，指定时间之内（1分钟、30分钟）限制IP访问次数（10次、50次）
     *
     * @param ip 客户端IP地址
     */
    public boolean isBlocked(String ip) {
        int[] num = {Const.ONE_MM, 3 * Const.TEN_MM, 10, 50};
        return isBlocked(ip, num);
    }

    /**
     * 对短时间内尝试登陆的ip次数进行限制
     *
     * @param ip  客户端IP地址
     * @param num 一定时间内及次数限制
     * @return boolean
     */
    public boolean isBlocked(String ip, int[] num) {
        String rk = Const.RDS_DENY + ip;
        Map<String, String> addr = hmget(rk);
        // 记录当前访问时间戳，如：1693814052623
        String time = String.valueOf(new Date().getTime());
        int loop = num.length / 2;
        if (addr == null) {
            // 初次访问，未锁
            addr = new HashMap<>();
            addr.put("locked", "0");
        } else if ("1".equals(addr.get("locked"))) { // 已锁定
            return true;
        } else {
            String[] arr = addr.get("time").split(",");
            int len = arr.length, i = 0;
            while (i < len) {
                long diff = new Date().getTime() - Long.parseLong(arr[i]);
                for (int n = 0; n < loop; n++) {
                    if (diff < num[0]) { // 指定时间之内（1分钟、30分钟）
                        if (len - i < num[n + loop]) // 小于N次
                            break;
                        // N次及以上（10次、50次）
                        addr.put("locked", "1"); // 锁定
                        hmset(rk, addr, loop - 1);
                        return true;
                    }
                }
                i++;
            }
            while (i < len) {
                time = arr[i] + "," + time;
                i++;
            }
        }
        addr.put("time", time);
        hmset(rk, addr, loop - 1); // 循环次数
        return false;
    }

	/**
	 * 生成一个66位国密SM2公钥，并缓存私钥 sec秒后失效
	 * <p>
	 * 仅登录初始化用在了getTicket()中，缓存2分钟
	 */
	public String getSm2PubKey(Object ticket, int sec) {
		Map<String, String> keyPair = Digest.genSm2Pair();
		String pubk = keyPair.get(Digest.PUB_KEY); // 66位公钥
		String prik = keyPair.get(Digest.PRI_KEY); // 64位私钥
		/**
		 * 用了两种方式来缓存私钥（sec秒后失效）<br>
		 * 1.以ticket为key暂存私钥<br>
		 * 2.以临时公钥为key暂存私钥
		 */
		set(Const.RDS_APPID + ticket, prik, sec);
		set(Const.RDS_CERT + pubk, prik, sec);
		/** 返回前端公钥 */
		return pubk;
	}

	/**
	 * 读取临时缓存的64位国密SM2私钥
	 */
//	public String getSm2PriKey(Object key) {
	String getCertKey(Object key) {
		/**
		 * 支持两种方式获取私钥<br>
		 * 1.以ticket为key获取私钥<br>
		 * 2.以临时公钥为key获取私钥（目前使用第2种方式）
		 */
		String prik = get(Const.RDS_APPID + key);
		return prik == null ? get(Const.RDS_CERT + key) : prik;
	}

}
